import { EventsKey } from '../events';
import BaseEvent from '../events/Event';
import MapEvent from '../MapEvent';
import { ObjectEvent } from '../Object';
import Control from './Control';

export interface Options {
    className?: string | undefined;
    target?: HTMLElement | string | undefined;
    collapsible?: boolean | undefined;
    collapsed?: boolean | undefined;
    tipLabel?: string | undefined;
    label?: string | HTMLElement | undefined;
    expandClassName?: string | undefined;
    collapseLabel?: string | HTMLElement | undefined;
    collapseClassName?: string | undefined;
    render?: ((p0: MapEvent) => void) | undefined;
}
export default class Attribution extends Control {
    constructor(opt_options?: Options);
    /**
     * Return true when the attribution is currently collapsed or false
     * otherwise.
     */
    getCollapsed(): boolean;
    /**
     * Return true if the attribution is collapsible, false otherwise.
     */
    getCollapsible(): boolean;
    /**
     * Collapse or expand the attribution according to the passed parameter. Will
     * not do anything if the attribution isn't collapsible or if the current
     * collapsed state is already the one requested.
     */
    setCollapsed(collapsed: boolean): void;
    /**
     * Set whether the attribution should be collapsible.
     */
    setCollapsible(collapsible: boolean): void;
    on(type: string | string[], listener: (p0: any) => any): EventsKey | EventsKey[];
    once(type: string | string[], listener: (p0: any) => any): EventsKey | EventsKey[];
    un(type: string | string[], listener: (p0: any) => any): void;
    on(type: 'change', listener: (evt: BaseEvent) => void): EventsKey;
    once(type: 'change', listener: (evt: BaseEvent) => void): EventsKey;
    un(type: 'change', listener: (evt: BaseEvent) => void): void;
    on(type: 'error', listener: (evt: BaseEvent) => void): EventsKey;
    once(type: 'error', listener: (evt: BaseEvent) => void): EventsKey;
    un(type: 'error', listener: (evt: BaseEvent) => void): void;
    on(type: 'propertychange', listener: (evt: ObjectEvent) => void): EventsKey;
    once(type: 'propertychange', listener: (evt: ObjectEvent) => void): EventsKey;
    un(type: 'propertychange', listener: (evt: ObjectEvent) => void): void;
}
